export declare type EventHandler = (event?: any) => void;
export declare type WildCardEventHandler = (type: string, event?: any) => void;
export declare type Handler = EventHandler | WildCardEventHandler;
export declare type EventHandlerMap = {
    [type: string]: Handler[];
};
export interface Emitter {
    on(type: string, handler: Handler): void;
    off(type: string, handler: Handler): void;
    emit(type: string, evt: any): void;
}
/** Mitt: Tiny (~200b) functional event emitter / pubsub.
 *  @name mitt
 *  @returns {Emitter}
 */
export declare function mitt(all?: EventHandlerMap): Emitter;
import { modelSymbol, parentSymbol, apc, vpc, modelName } from './symbols';
export declare type EmitFn = (evt: string, val: Event) => void;
export declare type ModelCommons = {
    onAction: (fn: EventHandler, after?: boolean) => Function;
    onSnapshot: (fn: EventHandler) => Function;
    onPatch: (fn: EventHandler) => Function;
    getSnapshot: () => object;
    applySnapshot: (snapshot: object, dontemit?: boolean) => void;
    getParent: () => Model<any> | null;
    getRoot: () => Model<any>;
};
export declare type FnMap = {
    [index: string]: Function;
};
export declare type Model<S> = ModelCommons & {
    [vpc]: Model<S>;
    [apc]: Model<S>;
    [modelName]: string;
    [modelSymbol]: boolean;
    [parentSymbol]: (emit: EmitFn, _parent: Model<S>, _path: string) => void;
    [index: string]: any;
} & S;
export declare type Event = {
    path?: string;
    value?: string;
    [index: string]: any;
};
export declare type ModelArgs<S> = {
    initial: () => S;
    actions?: (self: Model<S>) => FnMap;
    views?: (self: Model<S>) => FnMap;
};
declare function model<S>(name: string, {initial, actions, views}: ModelArgs<S>): (obj?: object | undefined) => Model<S>;
export default model;
export declare function clearCache(): void;
